@@ -1,11 +1,13 @@
 import json
 from re import match
+from functools import reduce
 from samtranslator.model import PropertyType, Resource
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.model.types import is_type, one_of, is_str, list_of
 from samtranslator.model.intrinsics import ref, fnSub
 from samtranslator.translator import logical_id_generator
 from samtranslator.translator.arn_generator import ArnGenerator
+from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
 
 
 class ApiGatewayRestApi(Resource):
@@ -128,15 +130,16 @@ class ApiGatewayResponse(object):
             raise InvalidResourceException(api_logical_id, "Property 'StatusCode' must be numeric")
 
         self.api_logical_id = api_logical_id
-        self.response_parameters = response_parameters or {}
-        self.response_templates = response_templates or {}
+        # Defaults to Py27Dict() as these will go into swagger
+        self.response_parameters = response_parameters or Py27Dict()
+        self.response_templates = response_templates or Py27Dict()
         self.status_code = status_code_str
 
     def generate_swagger(self):
-        swagger = {
-            "responseParameters": self._add_prefixes(self.response_parameters),
-            "responseTemplates": self.response_templates,
-        }
+        # Applying Py27Dict here as this goes into swagger
+        swagger = Py27Dict()
+        swagger["responseParameters"] = self._add_prefixes(self.response_parameters)
+        swagger["responseTemplates"] = self.response_templates
 
         # Prevent "null" being written.
         if self.status_code:
@@ -146,13 +149,17 @@ class ApiGatewayResponse(object):
 
     def _add_prefixes(self, response_parameters):
         GATEWAY_RESPONSE_PREFIX = "gatewayresponse."
-        prefixed_parameters = {}
-        for key, value in response_parameters.get("Headers", {}).items():
-            prefixed_parameters[GATEWAY_RESPONSE_PREFIX + "header." + key] = value
-        for key, value in response_parameters.get("Paths", {}).items():
-            prefixed_parameters[GATEWAY_RESPONSE_PREFIX + "path." + key] = value
-        for key, value in response_parameters.get("QueryStrings", {}).items():
-            prefixed_parameters[GATEWAY_RESPONSE_PREFIX + "querystring." + key] = value
+        # applying Py27Dict as this is part of swagger
+        prefixed_parameters = Py27Dict()
+
+        parameter_prefix_pairs = [("Headers", "header."), ("Paths", "path."), ("QueryStrings", "querystring.")]
+        for parameter, prefix in parameter_prefix_pairs:
+            for key, value in response_parameters.get(parameter, {}).items():
+                param_key = GATEWAY_RESPONSE_PREFIX + prefix + key
+                if isinstance(key, Py27UniStr):
+                    # if key is from template, we need to convert param_key to Py27UniStr
+                    param_key = Py27UniStr(param_key)
+                prefixed_parameters[param_key] = value
 
         return prefixed_parameters
 
@@ -278,7 +285,7 @@ class ApiGatewayAuthorizer(object):
         try:
             ttl_int = int(ttl)
         # this will catch if ttl is None and not convertable to an int
-        except TypeError:
+        except (TypeError, ValueError):
             # previous behavior before trying to read ttl
             return required_properties_missing
 
@@ -288,21 +295,20 @@ class ApiGatewayAuthorizer(object):
     def generate_swagger(self):
         authorizer_type = self._get_type()
         APIGATEWAY_AUTHORIZER_KEY = "x-amazon-apigateway-authorizer"
-        swagger = {
-            "type": "apiKey",
-            "name": self._get_swagger_header_name(),
-            "in": "header",
-            "x-amazon-apigateway-authtype": self._get_swagger_authtype(),
-        }
+        swagger = Py27Dict()
+        swagger["type"] = "apiKey"
+        swagger["name"] = self._get_swagger_header_name()
+        swagger["in"] = "header"
+        swagger["x-amazon-apigateway-authtype"] = self._get_swagger_authtype()
 
         if authorizer_type == "COGNITO_USER_POOLS":
-            swagger[APIGATEWAY_AUTHORIZER_KEY] = {
-                "type": self._get_swagger_authorizer_type(),
-                "providerARNs": self._get_user_pool_arn_array(),
-            }
+            authorizer_dict = Py27Dict()
+            authorizer_dict["type"] = self._get_swagger_authorizer_type()
+            authorizer_dict["providerARNs"] = self._get_user_pool_arn_array()
+            swagger[APIGATEWAY_AUTHORIZER_KEY] = authorizer_dict
 
         elif authorizer_type == "LAMBDA":
-            swagger[APIGATEWAY_AUTHORIZER_KEY] = {"type": self._get_swagger_authorizer_type()}
+            swagger[APIGATEWAY_AUTHORIZER_KEY] = Py27Dict({"type": self._get_swagger_authorizer_type()})
             partition = ArnGenerator.get_partition_name()
             resource = "lambda:path/2015-03-31/functions/${__FunctionArn__}/invocations"
             authorizer_uri = fnSub(
@@ -341,35 +347,39 @@ class ApiGatewayAuthorizer(object):
     def _get_identity_validation_expression(self):
         return self.identity and self.identity.get("ValidationExpression")
 
+    def _build_identity_source_item(self, item_prefix, prop_value):
+        item = item_prefix + prop_value
+        if isinstance(prop_value, Py27UniStr):
+            item = Py27UniStr(item)
+        return item
+
+    def _build_identity_source_item_array(self, prop_key, item_prefix):
+        arr = []
+        if self.identity.get(prop_key):
+            arr = [
+                self._build_identity_source_item(item_prefix, prop_value) for prop_value in self.identity.get(prop_key)
+            ]
+        return arr
+
     def _get_identity_source(self):
-        identity_source_headers = []
-        identity_source_query_strings = []
-        identity_source_stage_variables = []
-        identity_source_context = []
+        key_prefix_pairs = [
+            ("Headers", "method.request.header."),
+            ("QueryStrings", "method.request.querystring."),
+            ("StageVariables", "stageVariables."),
+            ("Context", "context."),
+        ]
 
-        if self.identity.get("Headers"):
-            identity_source_headers = list(map(lambda h: "method.request.header." + h, self.identity.get("Headers")))
-
-        if self.identity.get("QueryStrings"):
-            identity_source_query_strings = list(
-                map(lambda qs: "method.request.querystring." + qs, self.identity.get("QueryStrings"))
-            )
-
-        if self.identity.get("StageVariables"):
-            identity_source_stage_variables = list(
-                map(lambda sv: "stageVariables." + sv, self.identity.get("StageVariables"))
-            )
-
-        if self.identity.get("Context"):
-            identity_source_context = list(map(lambda c: "context." + c, self.identity.get("Context")))
-
-        identity_source_array = (
-            identity_source_headers
-            + identity_source_query_strings
-            + identity_source_stage_variables
-            + identity_source_context
+        identity_source_array = reduce(
+            lambda accumulator, key_prefix_pair: accumulator
+            + self._build_identity_source_item_array(key_prefix_pair[0], key_prefix_pair[1]),
+            key_prefix_pairs,
+            [],
         )
+
         identity_source = ", ".join(identity_source_array)
+        if any(isinstance(i, Py27UniStr) for i in identity_source_array):
+            # Convert identity_source to Py27UniStr if any part of it is Py27UniStr
+            identity_source = Py27UniStr(identity_source)
 
         return identity_source
 
@@ -395,6 +405,13 @@ class ApiGatewayAuthorizer(object):
         return "LAMBDA"
 
     def _get_identity_header(self):
+        if self.identity and not isinstance(self.identity, dict):
+            raise InvalidResourceException(
+                self.api_logical_id,
+                "Auth.Authorizers.<Authorizer>.Identity must be a dict (LambdaTokenAuthorizationIdentity, "
+                "LambdaRequestAuthorizationIdentity or CognitoAuthorizationIdentity).",
+            )
+
         if not self.identity or not self.identity.get("Header"):
             return "Authorization"
 
