@@ -1,6 +1,5 @@
 import copy
 import re
-from six import string_types
 
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model import ResourceMacro, PropertyType
@@ -23,6 +22,7 @@ from samtranslator.translator.arn_generator import ArnGenerator
 from samtranslator.model.exceptions import InvalidEventException, InvalidResourceException
 from samtranslator.swagger.swagger import SwaggerEditor
 from samtranslator.open_api.open_api import OpenApiEditor
+from samtranslator.utils.py27hash_fix import Py27Dict, Py27UniStr
 
 CONDITION = "Condition"
 
@@ -256,7 +256,7 @@ class S3(PushEventSource):
     def resources_to_link(self, resources):
         if isinstance(self.Bucket, dict) and "Ref" in self.Bucket:
             bucket_id = self.Bucket["Ref"]
-            if not isinstance(bucket_id, string_types):
+            if not isinstance(bucket_id, str):
                 raise InvalidEventException(self.relative_id, "'Ref' value in S3 events is not a valid string.")
             if bucket_id in resources:
                 return {"bucket": resources[bucket_id], "bucket_id": bucket_id}
@@ -325,7 +325,7 @@ class S3(PushEventSource):
         depends_on = bucket.get("DependsOn", [])
 
         # DependsOn can be either a list of strings or a scalar string
-        if isinstance(depends_on, string_types):
+        if isinstance(depends_on, str):
             depends_on = [depends_on]
 
         try:
@@ -376,7 +376,7 @@ class S3(PushEventSource):
             base_event_mapping["Filter"] = self.Filter
 
         event_types = self.Events
-        if isinstance(self.Events, string_types):
+        if isinstance(self.Events, str):
             event_types = [self.Events]
 
         event_mappings = []
@@ -403,6 +403,9 @@ class S3(PushEventSource):
             lambda_notifications = []
             notification_config["LambdaConfigurations"] = lambda_notifications
 
+        if not isinstance(lambda_notifications, list):
+            raise InvalidResourceException(self.logical_id, "Invalid type for LambdaConfigurations. Must be a list.")
+
         for event_mapping in event_mappings:
             if event_mapping not in lambda_notifications:
                 lambda_notifications.append(event_mapping)
@@ -556,7 +559,7 @@ class Api(PushEventSource):
         stage_suffix = "AllStages"
         explicit_api = None
         rest_api_id = self.get_rest_api_id_string(self.RestApiId)
-        if isinstance(rest_api_id, string_types):
+        if isinstance(rest_api_id, str):
 
             if (
                 rest_api_id in resources
@@ -568,7 +571,7 @@ class Api(PushEventSource):
                 permitted_stage = explicit_api["StageName"]
 
                 # Stage could be a intrinsic, in which case leave the suffix to default value
-                if isinstance(permitted_stage, string_types):
+                if isinstance(permitted_stage, str):
                     if not permitted_stage:
                         raise InvalidResourceException(rest_api_id, "StageName cannot be empty.")
                     stage_suffix = permitted_stage
@@ -663,15 +666,8 @@ class Api(PushEventSource):
         if swagger_body is None:
             return
 
-        function_arn = function.get_runtime_attr("arn")
         partition = ArnGenerator.get_partition_name()
-        uri = fnSub(
-            "arn:"
-            + partition
-            + ":apigateway:${AWS::Region}:lambda:path/2015-03-31/functions/"
-            + make_shorthand(function_arn)
-            + "/invocations"
-        )
+        uri = _build_apigw_integration_uri(function, partition)
 
         editor = SwaggerEditor(swagger_body)
 
@@ -708,14 +704,9 @@ class Api(PushEventSource):
                             ),
                         )
 
-                    if not isinstance(method_authorizer, string_types):
-                        raise InvalidEventException(
-                            self.relative_id,
-                            "Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}] "
-                            "because it wasn't defined with acceptable values in the API's Authorizers.".format(
-                                authorizer=method_authorizer, method=self.Method, path=self.Path
-                            ),
-                        )
+                    _check_valid_authorizer_types(
+                        self.relative_id, self.Method, self.Path, method_authorizer, api_authorizers
+                    )
 
                     if method_authorizer != "NONE" and not api_authorizers.get(method_authorizer):
                         raise InvalidEventException(
@@ -776,7 +767,7 @@ class Api(PushEventSource):
                             model=method_model, method=self.Method, path=self.Path
                         ),
                     )
-                if not isinstance(method_model, string_types):
+                if not isinstance(method_model, str):
                     raise InvalidEventException(
                         self.relative_id,
                         "Unable to set RequestModel [{model}] on API method [{method}] for path [{path}] "
@@ -838,7 +829,7 @@ class Api(PushEventSource):
 
                     parameter_name, parameter_value = next(iter(parameter.items()))
 
-                    if not re.match("method\.request\.(querystring|path|header)\.", parameter_name):
+                    if not re.match(r"method\.request\.(querystring|path|header)\.", parameter_name):
                         raise InvalidEventException(
                             self.relative_id,
                             "Invalid value for 'RequestParameters' property. Keys must be in the format "
@@ -861,8 +852,8 @@ class Api(PushEventSource):
 
                     parameters.append(settings)
 
-                elif isinstance(parameter, string_types):
-                    if not re.match("method\.request\.(querystring|path|header)\.", parameter):
+                elif isinstance(parameter, str):
+                    if not re.match(r"method\.request\.(querystring|path|header)\.", parameter):
                         raise InvalidEventException(
                             self.relative_id,
                             "Invalid value for 'RequestParameters' property. Keys must be in the format "
@@ -976,7 +967,7 @@ class Cognito(PushEventSource):
     def resources_to_link(self, resources):
         if isinstance(self.UserPool, dict) and "Ref" in self.UserPool:
             userpool_id = self.UserPool["Ref"]
-            if not isinstance(userpool_id, string_types):
+            if not isinstance(userpool_id, str):
                 raise InvalidEventException(
                     self.logical_id,
                     "Ref in Userpool is not a string.",
@@ -1018,7 +1009,7 @@ class Cognito(PushEventSource):
 
     def _inject_lambda_config(self, function, userpool):
         event_triggers = self.Trigger
-        if isinstance(self.Trigger, string_types):
+        if isinstance(self.Trigger, str):
             event_triggers = [self.Trigger]
 
         # TODO can these be conditional?
@@ -1069,7 +1060,7 @@ class HttpApi(PushEventSource):
         if isinstance(api_id, dict) and "Ref" in api_id:
             api_id = api_id["Ref"]
 
-        explicit_api = resources[api_id].get("Properties")
+        explicit_api = resources[api_id].get("Properties", {})
 
         return {"explicit_api": explicit_api}
 
@@ -1127,8 +1118,8 @@ class HttpApi(PushEventSource):
         # If this is using the new $default path, keep path blank and add a * permission
         if path == OpenApiEditor._DEFAULT_PATH:
             path = ""
-        elif editor and resources_to_link.get("function").logical_id == editor.get_integration_function_logical_id(
-            OpenApiEditor._DEFAULT_PATH, OpenApiEditor._X_ANY_METHOD
+        elif editor and editor.is_integration_function_logical_id_match(
+            OpenApiEditor._DEFAULT_PATH, OpenApiEditor._X_ANY_METHOD, resources_to_link.get("function").logical_id
         ):
             # Case where default exists for this function, and so the permissions for that will apply here as well
             # This can save us several CFN resources (not duplicating permissions)
@@ -1162,12 +1153,7 @@ class HttpApi(PushEventSource):
         if open_api_body is None:
             return
 
-        function_arn = function.get_runtime_attr("arn")
-        uri = fnSub(
-            "arn:${AWS::Partition}:apigateway:${AWS::Region}:lambda:path/2015-03-31/functions/"
-            + make_shorthand(function_arn)
-            + "/invocations"
-        )
+        uri = _build_apigw_integration_uri(function, "${AWS::Partition}")
 
         editor = OpenApiEditor(open_api_body)
 
@@ -1223,6 +1209,8 @@ class HttpApi(PushEventSource):
         # Default auth should already be applied, so apply any other auth here or scope override to default
         api_authorizers = api_auth and api_auth.get("Authorizers")
 
+        _check_valid_authorizer_types(self.relative_id, self.Method, self.Path, method_authorizer, api_authorizers)
+
         if method_authorizer != "NONE" and not api_authorizers:
             raise InvalidEventException(
                 self.relative_id,
@@ -1256,3 +1244,35 @@ class HttpApi(PushEventSource):
                 "'AuthorizationScopes' must be a list of strings.".format(method=self.Method, path=self.Path),
             )
         editor.add_auth_to_method(api=api, path=self.Path, method_name=self.Method, auth=self.Auth)
+
+
+def _build_apigw_integration_uri(function, partition):
+    function_arn = function.get_runtime_attr("arn")
+    arn = (
+        "arn:"
+        + partition
+        + ":apigateway:${AWS::Region}:lambda:path/2015-03-31/functions/"
+        + make_shorthand(function_arn)
+        + "/invocations"
+    )
+    # function_arn is always of the form {"Fn::GetAtt": ["<function_logical_id>", "Arn"]}.
+    # We only want to check if the function logical id is a Py27UniStr instance.
+    if function_arn.get("Fn::GetAtt") and isinstance(function_arn["Fn::GetAtt"][0], Py27UniStr):
+        arn = Py27UniStr(arn)
+    return Py27Dict(fnSub(arn))
+
+
+def _check_valid_authorizer_types(relative_id, method, path, method_authorizer, api_authorizers):
+    if method_authorizer == "NONE":
+        # If the method authorizer is "NONE" then this check
+        # isn't needed since DefaultAuthorizer needs to be used.
+        return
+
+    if not isinstance(method_authorizer, str) or not isinstance(api_authorizers, dict):
+        raise InvalidEventException(
+            relative_id,
+            "Unable to set Authorizer [{authorizer}] on API method [{method}] for path [{path}]. "
+            "The method authorizer must be a string with a corresponding dict entry in the api authorizer.".format(
+                authorizer=method_authorizer, method=method, path=path
+            ),
+        )
