@@ -1,7 +1,6 @@
 ﻿from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.public.intrinsics import is_intrinsics
 from samtranslator.swagger.swagger import SwaggerEditor
-from six import string_types
 
 
 class Globals(object):
@@ -43,6 +42,8 @@ class Globals(object):
             "FileSystemConfigs",
             "CodeSigningConfigArn",
             "Architectures",
+            "EphemeralStorage",
+            "FunctionUrlConfig",
         ],
         # Everything except
         #   DefinitionBody: because its hard to reason about merge of Swagger dictionaries
@@ -80,6 +81,8 @@ class Globals(object):
         ],
         SamResourceType.SimpleTable.value: ["SSESpecification"],
     }
+    # unreleased_properties *must be* part of supported_properties too
+    unreleased_properties = {}
 
     def __init__(self, template):
         """
@@ -156,7 +159,7 @@ class Globals(object):
                         SwaggerEditor.get_openapi_version_3_regex(), properties[cls._OPENAPIVERSION]
                     )
                 ):
-                    if not isinstance(properties[cls._OPENAPIVERSION], string_types):
+                    if not isinstance(properties[cls._OPENAPIVERSION], str):
                         properties[cls._OPENAPIVERSION] = str(properties[cls._OPENAPIVERSION])
                         resource["Properties"] = properties
                     if "DefinitionBody" in properties:
@@ -195,14 +198,17 @@ class Globals(object):
             if not isinstance(properties, dict):
                 raise InvalidGlobalsSectionException(self._KEYWORD, "Value of ${section} must be a dictionary")
 
+            supported = self.supported_properties[resource_type]
+            supported_displayed = [
+                prop for prop in supported if prop not in self.unreleased_properties.get(resource_type, [])
+            ]
             for key, value in properties.items():
-                supported = self.supported_properties[resource_type]
                 if key not in supported:
                     raise InvalidGlobalsSectionException(
                         self._KEYWORD,
                         "'{key}' is not a supported property of '{section}'. "
                         "Must be one of the following values - {supported}".format(
-                            key=key, section=section_name, supported=supported
+                            key=key, section=section_name, supported=supported_displayed
                         ),
                     )
 
